/*
 * interface.c
 * 
 * This file will include all the interfaces between the input routines and the
 * daemon.  They should be useful for writing robots and the like
 */
#include "copyright.h"

#include <stdio.h>
#include <math.h>
#include <signal.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"
#ifndef _NO_DEPEND
#   include "ntserv_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern char *                 strcat P_((char *, const char *));

#undef P_
#endif

void
set_speed(speed)
   int             speed;
{
   if(speed == me->p_desspeed) return;

   if (speed > me->p_ship.s_maxspeed) {
      me->p_desspeed = me->p_ship.s_maxspeed;
   } else if (speed < 0) {
      speed = 0;
   }
   me->p_desspeed = speed;
   if (me->p_flags & PFDOCK) {
#ifdef SB_DOCK
      players[me->p_docked].p_numdocked--;
      players[me->p_docked].p_port[me->p_portdocked] = VACANT;
#else
      players[me->p_docked].p_docked--;
      players[me->p_docked].p_port[me->p_port[0]] = VACANT;
#endif
   }
   me->p_flags &= ~(PFREPAIR | PFBOMB | PFORBIT | PFDOCK | PFBEAMUP | PFBEAMDOWN);
}

void
set_course(dir)
   unsigned char   dir;
{
   me->p_desdir = dir;
   if (me->p_flags & PFDOCK) {
#ifdef SB_DOCK
      players[me->p_docked].p_numdocked--;
      players[me->p_docked].p_port[me->p_portdocked] = VACANT;
#else
      players[me->p_docked].p_docked--;
      players[me->p_docked].p_port[me->p_port[0]] = VACANT;
#endif
   }
   me->p_flags &= ~(PFBOMB | PFORBIT | PFDOCK | PFBEAMUP | PFBEAMDOWN);
}

void
shield_up()
{
   me->p_flags |= PFSHIELD;
   me->p_flags &= ~(PFBOMB | PFREPAIR | PFBEAMUP | PFBEAMDOWN);
}

void
shield_down()
{
   me->p_flags &= ~PFSHIELD;
}

void
shield_tog()
{
   me->p_flags ^= PFSHIELD;
   me->p_flags &= ~(PFBOMB | PFREPAIR | PFBEAMUP | PFBEAMDOWN);
}

void
bomb_planet()
{
   if (!(me->p_flags & PFORBIT)) {
      warning(0, "Must be orbiting to bomb", 39);
      return;
   }
   me->p_flags |= PFBOMB;
   me->p_flags &= ~(PFSHIELD | PFREPAIR | PFBEAMUP | PFBEAMDOWN);
}

void
beam_up()
{
   if (!(me->p_flags & (PFORBIT | PFDOCK))) {
      warning(0, "Must be orbiting or docked to beam up.", 44);
      return;
   }
   if (me->p_flags & PFORBIT) {
      if (me->p_team != planets[me->p_planet].pl_owner) {
	 warning(1, "Those aren't our men.", 45);
	 return;
      }
   } else if (me->p_flags & PFDOCK) {
      if (me->p_team != players[me->p_docked].p_team) {
	 warning(1, "Comm Officer: We're not authorized to beam foriegn troops on board!", 46);
	 return;
      }
   }
   me->p_flags |= PFBEAMUP;
   me->p_flags &= ~(PFSHIELD | PFREPAIR | PFBOMB | PFBEAMDOWN);
}

void
beam_down()
{
   if (!(me->p_flags & (PFORBIT | PFDOCK))) {
      warning(0, "Must be orbiting or docked to beam down.", 47);
      return;
   }
   if (me->p_flags & PFDOCK) {
      if (me->p_team != players[me->p_docked].p_team) {
	 warning(1, "Comm Officer: Starbase refuses permission to beam our troops over.", 48);
	 return;
      }
   }
   me->p_flags |= PFBEAMDOWN;
   me->p_flags &= ~(PFSHIELD | PFREPAIR | PFBOMB | PFBEAMUP);
}

void
repair()
{
   me->p_desspeed = 0;
   me->p_flags |= PFREPAIR;
   me->p_flags &= ~(PFSHIELD | PFBOMB | PFBEAMUP | PFBEAMDOWN | PFPLOCK | PFPLLOCK);
}

void
repair_off()
{
   me->p_flags &= ~PFREPAIR;
}

void
repeat_message()
{
   if (++lastm == MAXMESSAGE);
   lastm = 0;
}

void
cloak()
{
   me->p_flags ^= PFCLOAK;
   me->p_flags &= ~(PFTRACT | PFPRESS);
}

void
cloak_on()
{
   me->p_flags |= PFCLOAK;
   me->p_flags &= ~(PFTRACT | PFPRESS);
}

void
cloak_off()
{
   me->p_flags &= ~PFCLOAK;
}

void
lock_planet(planet)
   int             planet;
{
   char            buf[80];

   if((me->p_flags & PFPLLOCK ) && (me->p_planet == planet))
      return;

   if (planet < 0 || planet >= MAXPLANETS)
      return;

   me->p_flags |= PFPLLOCK;
   me->p_flags &= ~(PFPLOCK | PFORBIT | PFBEAMUP | PFBEAMDOWN | PFBOMB);
   me->p_planet = planet;
#ifdef SHORT_PACKETS
   if(send_short){
      swarning(LOCKPLANET_TEXT, (unsigned char) planet, 0);
      return;
   }
#endif
   sprintf(buf, "Locking onto %s", planets[planet].pl_name);
   warning(0, buf, -1);
}

void
lock_player(player)
   int             player;
{
   char            buf[80];

   if((me->p_flags & PFPLOCK ) && (me->p_playerl == player))
      return;

   if (player < 0 || player >= MAXPLAYER)
      return;
   if (players[player].p_status != PALIVE)
      return;
   if (players[player].p_flags & PFCLOAK)
      return;

   me->p_flags |= PFPLOCK;
   me->p_flags &= ~(PFPLLOCK | PFORBIT | PFBEAMUP | PFBEAMDOWN | PFBOMB);
   me->p_playerl = player;
#ifdef SHORT_PACKETS
   if(send_short){
      /*
      swarning(LOCKPLAYER_TEXT, (unsigned char) player, 0);
      */
      swarning(SBLOCKSTRANGER, (unsigned char) player, 0);
      return;
   }
#endif
   sprintf(buf, "Locking onto %s (%c%c)",
	   players[player].p_name,
	   teamlet[players[player].p_team],
	   shipnos[players[player].p_no]);
   if (players[player].p_ship.s_type == STARBASE) {
      if ((players[player].p_flags & PFDOCKOK))
	 strcat(buf, " (docking permission ENABLED)");
      else
	 strcat(buf, " (docking permission DENIED)");
   }
   warning(0, buf, -1);
}

void
declare_war(mask)
   int             mask;
{
   int             changes;
   int             i;

   mask &= ALLTEAM;
   mask &= ~me->p_team;
   mask |= me->p_swar;
   changes = mask ^ me->p_hostile;
   if (changes == 0)
      return;

   if (changes & FED) {
      sendwarn("Federation", mask & FED, FED);
   }
   if (changes & ROM) {
      sendwarn("Romulans", mask & ROM, ROM);
   }
   if (changes & KLI) {
      sendwarn("Klingons", mask & KLI, KLI);
   }
   if (changes & ORI) {
      sendwarn("Orions", mask & ORI, ORI);
   }
   if (me->p_flags & PFDOCK) {
      if (players[me->p_docked].p_team & mask) {
#ifdef SB_DOCK
	 players[me->p_docked].p_port[me->p_portdocked] = VACANT;
	 players[me->p_docked].p_numdocked--;
#else
	 players[me->p_docked].p_port[me->p_port[0]] = VACANT;
	 players[me->p_docked].p_docked--;
#endif
	 me->p_flags &= ~PFDOCK;
      }
   } else if (me->p_ship.s_type == STARBASE) {
      if (
#ifdef SB_DOCK
	  me->p_numdocked > 0
#else
	  me->p_docked > 0
#endif
	 ) {
	 for (i = 0; i < NUMPORTS; i++) {
	    if (me->p_port[i] == VACANT)	/* fix -- TSH */
	       continue;
	    if (mask & players[me->p_port[i]].p_team) {
	       players[me->p_port[i]].p_flags &= ~PFDOCK;
#ifdef SB_DOCK
	       me->p_numdocked--;
#else
	       me->p_docked--;
#endif
	       me->p_port[i] = VACANT;
	    }
	 }
      }
   }
   if (mask & ~me->p_hostile) {
      me->p_flags |= PFWAR;
      delay = me->p_updates + 100;
      warning(1, "Pausing ten seconds to re-program battle computers.", 49);
   }
   me->p_hostile = mask;
}

void
sendwarn(string, atwar, team)
   char           *string;
   int             atwar;
   int             team;
{
   char            buf[80];		/* was BUFSIZ */
   char            addrbuf[10];

   if (atwar) {
      (void) sprintf(buf, "%s (%c%c) declaring war on the %s",
		     me->p_name,
		     teamlet[me->p_team],
		     shipnos[me->p_no],
		     string);
   } else {
      (void) sprintf(buf, "%s (%c%c) declaring peace with the %s",
		     me->p_name,
		     teamlet[me->p_team],
		     shipnos[me->p_no],
		     string);
   }

   (void) sprintf(addrbuf, " %c%c->%-3s",
		  teamlet[me->p_team],
		  shipnos[me->p_no],
		  teamshort[team]);
   pmessage2(buf, team, MTEAM, addrbuf, me->p_no);
}

void
do_refit(type)
   int             type;
{
   int             i = 0;
   char            ibuf[80];

   if (type < 0 || type >= ATT)
      return;
   if (me->p_flags & PFORBIT) {
      if (!(planets[me->p_planet].pl_flags & PLHOME)) {
	 warning(1, "You must orbit your HOME planet to apply for command reassignment!", 50);
	 return;
      } else {
	 if (!(planets[me->p_planet].pl_flags & me->p_team)) {
	    warning(1, "You must orbit your home planet to apply for command reassignment!", 51);
	    return;
	 }
      }				/* if (PLHOME) */
   } else if (me->p_flags & PFDOCK) {
      if (type == STARBASE
#ifdef SB_DOCK
	  && !sb_dock
#endif
	 ) {
	 warning(1, "Can only refit to starbase on your home planet.", 52);
	 return;
      }
      if (players[me->p_docked].p_team != me->p_team) {
	 warning(1, "You must dock YOUR starbase to apply for command reassignment!", 53);
	 return;
      }
   } else {
      warning(1, "Must orbit home planet or dock your starbase to apply for command reassignment!", 54);
      return;
   }

   if ((me->p_damage > ((float) me->p_ship.s_maxdamage) * .75) ||
       (me->p_shield < ((float) me->p_ship.s_maxshield) * .75) ||
       (me->p_fuel < ((float) me->p_ship.s_maxfuel) * .75)) {
      warning(1, "Central Command refuses to accept a ship in this condition!",
	      55);
      return;
   }
   if ((me->p_armies > 0)) {
      warning(1, "You must beam your armies down before moving to your new ship", 56);
      return;
   }
   if (shipsallowed[type] == 0) {
      warning(1, "That ship hasn't been designed yet.", 57);
      return;
   }
   if (type == STARBASE) {
      if (me->p_stats.st_rank < sbrank) {
	 char            buf[100];
#ifdef SHORT_PACKETS
	 if(send_short){
	    swarning(SBRANK_TEXT, sbrank, 0);
	    return;
	 }
#endif
	 sprintf(buf, "You need a rank of %s or higher to command a starbase!", ranks[sbrank].name);
	 warning(1, buf, -1);
	 return;
      }
   }
   if (type == STARBASE && !chaos) {
      for (i = 0; i < MAXPLAYER; i++)
	 if ((me->p_no != i) &&
	     (players[i].p_status == PALIVE) &&
	     (players[i].p_team == me->p_team))
	    if (players[i].p_ship.s_type == STARBASE) {
	       warning(1, "Your side already has a starbase!", 58);
	       return;
	    }
   }
   if (type == STARBASE && !chaos) {
      if (numShips(me->p_team) < 4) {
	 warning(1, "Your team is not capable of defending such an expensive ship", -1);
	 return;
      }
   }
   if (type == STARBASE && !chaos) {
      if (teams[me->p_team].s_turns > 0) {
	 int             min;
	 /*
	  * tell how much time remains til starbase is ready. note that
	  * PLANETFUSE determines when s_turns is decremented It is currently
	  * set at 400 -- 20 minutes for reconstruction
	  */
	 /*
	  * update is in micro-secs -- convert to sec and force round up XX -
	  * this will wrap if UPDATE becomes greater then 400000
	  */
	 min = ((UPDATE * teams[1 << teamPick].s_turns * 400) / 1000000 + 30)
	    / 60;

	 sprintf(ibuf,
	      "Your new starbase requires %d more %s of construction time.",
		 min, min > 1 ? "minutes" : "minute");
	 warning(1, ibuf, -1);
	 return;
      }
   }
   if (type == STARBASE && numPlanets(me->p_team) < 5 && !chaos) {
      warning(1, "Your team's stuggling economy cannot support such an expenditure!", 61);
      return;
   }
   if ((me->p_ship.s_type == STARBASE) && (type != STARBASE)) {
      /* Reset kills to 0.0 */
      me->p_kills = 0;
      /* bump all docked ships */
      for (i = 0; i < NUMPORTS; i++)
	 if (me->p_port[i] != VACANT) {
	    players[me->p_port[i]].p_flags &= ~PFDOCK;
#ifdef SB_DOCK
	    me->p_numdocked--;
#else
	    me->p_docked--;
#endif
	    me->p_port[i] = VACANT;
	 }
   }
   getship(&(me->p_ship), type);
   if (type != STARBASE && me->p_kills < plkills) {
      me->p_ship.s_plasmacost = -1;
   }
   me->p_shield = me->p_ship.s_maxshield;
   me->p_damage = 0;
   me->p_fuel = me->p_ship.s_maxfuel;
   me->p_wtemp = 0;
   me->p_etemp = 0;
   if (type == STARBASE) {
#ifdef BASEPRACTICE
#ifndef SB_DOCK
      if (me->p_flags & PFDOCK) {
	 players[me->p_docked].p_docked--;
	 players[me->p_docked].p_port[me->p_port[0]] = VACANT;
	 me->p_flags &= ~PFDOCK;
      }
#endif
#endif

#ifdef SB_DOCK
      me->p_numdocked = 0;
#else
      me->p_docked = 0;
#endif
      for (i = 0; i < 4; i++)
	 me->p_port[i] = VACANT;
   }
   me->p_flags &= ~(PFREFIT);
   me->p_flags |= PFREFITTING;
   rdelay = me->p_updates + 50;

   warning(1, "You are being transported to your new vessel .... ", 62);
}

int
numShips(owner)
   int             owner;
{
   int             i, num = 0;
   struct player  *p;

   for (i = 0, p = players; i < MAXPLAYER; i++, p++) {
      if (p->p_status == PALIVE && p->p_team == owner) {
	 num++;
      }
   }
   return (num);
}

int
numPlanets(owner)
   int             owner;
{
   int             i, num = 0;
   struct planet  *p;

   for (i = 0, p = planets; i < MAXPLANETS; i++, p++) {
      if (p->pl_owner == owner) {
	 num++;
      }
   }
   return (num);
}


void
tractor_player(player)
   int             player;
{
   struct player  *victim;

   if (weaponsallowed[WP_TRACTOR] == 0) {
      return;
   }
   if ((player < 0) || (player > MAXPLAYER)) {	/* out of bounds = cancel */
      me->p_flags &= ~(PFTRACT | PFPRESS);
      return;
   }
   if (me->p_flags & PFCLOAK) {
      return;
   }
   if (player == me->p_no)
      return;
   victim = &players[player];
   if (victim->p_flags & PFCLOAK)
      return;
   if (hypot((double) me->p_x - victim->p_x,
	     (double) me->p_y - victim->p_y) <
       ((double) TRACTDIST) * me->p_ship.s_tractrng) {
      if (victim->p_flags & PFDOCK) {
#ifdef SB_DOCK
	 players[victim->p_docked].p_port[victim->p_portdocked] = VACANT;
	 players[victim->p_docked].p_numdocked--;
#else
	 players[victim->p_docked].p_port[victim->p_port[0]] = VACANT;
	 players[victim->p_docked].p_docked--;
#endif
      }
      if (me->p_flags & PFDOCK) {
#ifdef SB_DOCK
	 players[me->p_docked].p_numdocked--;
	 players[me->p_docked].p_port[me->p_portdocked] = VACANT;
#else
	 players[me->p_docked].p_docked--;
	 players[me->p_docked].p_port[me->p_port[0]] = VACANT;
#endif
      }
      victim->p_flags &= ~(PFORBIT | PFDOCK);
      me->p_flags &= ~(PFORBIT | PFDOCK);
      me->p_tractor = player;
      me->p_flags |= PFTRACT;
   }
}

void
pressor_player(player)
   int             player;
{
   int             target;
   struct player  *victim;

   if (me->p_flags & PFPRESS)
      return;

   if (weaponsallowed[WP_TRACTOR] == 0) {
      warning(1, "Tractor beams haven't been invented yet.", 0);
      return;
   }
   target = player;

   if ((target < 0) || (target > MAXPLAYER)) {	/* out of bounds = cancel */
      me->p_flags &= ~(PFTRACT | PFPRESS);
      return;
   }
   if (me->p_flags & PFCLOAK) {
      warning(0, "Weapons's Officer:  Cannot tractor while cloaked, sir!", 1);
      return;
   }
   victim = &players[target];
   if (victim->p_flags & PFCLOAK)
      return;

   if (hypot((double) me->p_x - victim->p_x,
	     (double) me->p_y - victim->p_y) <
       ((double) TRACTDIST) * me->p_ship.s_tractrng) {
      if (victim->p_flags & PFDOCK) {
#ifdef SB_DOCK
	 players[victim->p_docked].p_port[victim->p_portdocked] = VACANT;
	 players[victim->p_docked].p_numdocked--;
#else
	 players[victim->p_docked].p_port[victim->p_port[0]] = VACANT;
	 players[victim->p_docked].p_docked--;
#endif
      }
      if (me->p_flags & PFDOCK) {
#ifdef SB_DOCK
	 players[me->p_docked].p_numdocked--;
	 players[me->p_docked].p_port[me->p_portdocked] = VACANT;
#else
	 players[me->p_docked].p_docked--;
	 players[me->p_docked].p_port[me->p_port[0]] = VACANT;
#endif
      }
      victim->p_flags &= ~(PFORBIT | PFDOCK);
      me->p_flags &= ~(PFORBIT | PFDOCK);
      me->p_tractor = target;
      me->p_flags |= (PFTRACT | PFPRESS);
   } else {
      warning(0, "Weapon's Officer:  Vessel is out of range of our tractor beam.", 2);
   }
}
