#ifdef MINES
/*
 * mine.c
 */
#include "copyright.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "defs.h"
#include "struct.h"
#include "data.h"

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

#undef P_
#endif

long            random();

void
mine()
{
   register int    i;
   register struct torp *k, *k1;

   if (me->p_flags & PFWEP) {
      warning(0, "Torpedo launch tubes have exceeded maximum safe temperature!");
      return;
   }

   /* each mine == 2 torps */
   if (me->p_ntorp >= MAXTORP - 1) {
      warning(0, "Our computers limit us to having 8 live torpedos at a time captain!");
      return;
   }
   /* a mine costs 4 * torpcost */
   if (me->p_fuel < minepow * myship->s_torpcost) {
      warning(0, "We don't have enough fuel for a mine!");
      return;
   }
   if (me->p_flags & PFREPAIR) {
      warning(1, "We cannot fire while our vessel is in repair mode.");
      return;
   }
   if ((me->p_cloakphase) && (me->p_ship.s_type != ATT)) {
      warning(0, "We are unable to fire while in cloak, captain!");
      return;
   }
   /* Ok, lay the mine */

   me->p_ntorp += 2;

   me->p_fuel -= minepow * myship->s_torpcost;
   me->p_wtemp += minepow * ((myship->s_torpcost / 10) - 10);

   for (i = me->p_no * MAXTORP, k = &torps[i];	/* Find a free torp */
	i < me->p_no * MAXTORP + MAXTORP; i++, k++) {
      if (k->t_status == TFREE)
	 break;
   }
   k1 = k;

   /* this is tuned to the torp bitmap of the client so... */
   /* 1 pixel per 20 spaces galactic map.  Each torp is 3 pixels width/height */

   /* - 2 pixel upper right */
   _mine(k, i, me->p_x - 40, me->p_y - 40);

   for (i = me->p_no * MAXTORP, k = &torps[i];	/* Find a free torp */
	i < me->p_no * MAXTORP + MAXTORP; i++, k++) {
      if (k->t_status == TFREE)
	 break;
   }

   /* + 2 pixel lower left */
   _mine(k, i, me->p_x + 40, me->p_y + 40);

   /* match the pair */

   k1->t_pair = k->t_no;
   k->t_pair = k1->t_no;
}

void
_mine(k, i, x, y)
   struct torp    *k;
   int             i;
   int             x, y;
{
   k->t_no = i;
   k->t_status = TSTRAIGHT;	/* they don't move anyway */
   k->t_owner = me->p_no;
   k->t_team = me->p_team;
   k->t_x = x;
   k->t_y = y;
   k->t_dir = 0;		/* they don't move */
   k->t_damage = (minepow / 2) * myship->s_torpdamage;	/* does twice damage */
   k->t_speed = 0;		/* they don't move */
   k->t_war = me->p_hostile | me->p_swar;
   /* CHANGE */
   k->t_fuse = 1;		/* they don't run out */
   k->t_turns = 0;		/* don't turn */

   k->t_mine = 1;		/* mark as a mine */
}

/* det all mines */
void
mine_det()
{
   register int    i;
   register struct torp *k;

   for (i = me->p_no * MAXTORP, k = &torps[i];
	i < me->p_no * MAXTORP + MAXTORP; i++, k++) {
      if (k->t_mine && k->t_status == TSTRAIGHT)
	 k->t_status = TOFF;
   }
}
#endif
