/*
 * phaser.c
 */
#include "copyright.h"

#include <stdio.h>
#include <math.h>
#include <signal.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"
#ifndef _NO_DEPEND
#   include "ntserv_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern void                   perror P_((const char *));

#undef P_
#endif

void
phaser(course)
   unsigned char   course;
{
   register int    i;
   register struct player *j, *target = NULL;
   register struct plasmatorp *target2 = NULL;
   register struct plasmatorp *k;
   register struct phaser *mine;
   int             whichtarget;
   int             target_x = 0, target_y = 0;
   unsigned char   dir;
   int             range, trange;
   int             maxangle;
   int             myphrange;
   char            buf[80];

   mine = &phasers[me->p_no];

   if (myship->s_phasercost < 0) {
      warning(1, "Weapons Officer:  This ship is not armed with phasers, captain!", 31);
      return;
   }
   if (mine->ph_status != PHFREE) {
      warning(0, "Phasers have not recharged", 32);
      return;
   }
   if (me->p_fuel < myship->s_phasercost) {
      warning(0, "Not enough fuel for phaser", 33);
      return;
   }
   if (me->p_flags & PFREPAIR) {
      warning(1, "Can't fire while repairing", 34);
      return;
   }
   if (me->p_flags & PFWEP) {
      warning(0, "Weapons overheated", 35);
      return;
   }
   if ((me->p_cloakphase > (CLOAK_PHASES - 1) - cloakdelay) &&
       (me->p_ship.s_type != ATT))
   {
      warning(0, "Cannot fire while cloaked", 36);
      return;
   }
   me->p_fuel -= myship->s_phasercost;
   me->p_wtemp += myship->s_phasercost / 10;
   mine->ph_dir = course;

   whichtarget = 0;
   range = 1000000;		/* Sufficiently big. */

   for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
      if ((j->p_status != PALIVE
#ifdef BIRD
	   && !(j->p_flags & PFBIRD)
#endif
	   ) || (j == me))
	 continue;
      if (
#ifdef BIRD
	  !(j->p_flags & PFBIRD) &&
#endif
	  ((!((j->p_swar | j->p_hostile) & me->p_team)) &&
	   (!((me->p_swar | me->p_hostile) & j->p_team))))
	 continue;
      /* atan2() / 3.14159 * 128. == atan2() * (128./3.14159) */
      dir = (unsigned char) nint(atan2((double) (j->p_x - me->p_x),
			     (double) (me->p_y - j->p_y)) * (128./M_PI));

      /*
      MATH_ERR_CHECK(j->p_x-me->p_x + me->p_y-j->p_y);
      */
      trange = (int) hypot((double) (j->p_x - me->p_x),
			   (double) (j->p_y - me->p_y));
      if (trange == 0)
	 trange = 1;
      /* atan() / 3.14159 * 128. == atan() * (128./3.14159) */
      maxangle = nint(atan((float) EXPDIST / trange) * (128./M_PI));

      if (angdist(dir, course) <= maxangle) {
	 if (range > trange) {
	    whichtarget = 1;
	    target = j;
	    range = trange;
	 }
      }
   }

   for (i = 0, k = &plasmatorps[i]; i < MAXPLASMA * MAXPLAYER; i++, k++) {
      if ((k->pt_status != PTMOVE) || (k->pt_owner == me->p_no))
	 continue;
      if ((!(k->pt_war & me->p_team)) &&
	  (!((me->p_swar | me->p_hostile) & k->pt_team)))
	 continue;
      dir = (unsigned char) nint(atan2((double) (k->pt_x - me->p_x),
			    (double) (me->p_y - k->pt_y)) * (128./M_PI));
      MATH_ERR_CHECK(k->pt_x-me->p_x + me->p_y-k->pt_y);
      trange = (int) hypot((double) (k->pt_x - me->p_x),
			   (double) (k->pt_y - me->p_y));
      if (trange == 0)
	 trange = 1;
      maxangle = nint(atan((float) EXPDIST / 4 / trange) * (128./M_PI));
      if (angdist(dir, course) <= (maxangle + 1)) {
	 if (range > trange) {
	    target_x = k->pt_x;
	    target_y = k->pt_y;
	    whichtarget = 2;
	    target2 = k;
	    range = trange;
	 }
      }
   }

   mine->ph_fuse = me->p_ship.s_phaserfuse;

   myphrange = PHASEDIST * me->p_ship.s_phaserdamage / 100;

   if ((whichtarget == 0) ||
       (range > myphrange)) {
      mine->ph_status = PHMISS;
      warning(0, "Phaser missed!!!", 37);
   } else if (whichtarget == 2) {
#ifdef SNAKEPATROL
      if (players[target2->pt_owner].p_flags & PFSNAKE)
	 warning(1, "You destroyed the reptilian monstrosity!", -1);
      else
#endif

	 warning(1, "You destroyed the plasma torpedo!", 38);
      mine->ph_x = target_x;
      mine->ph_y = target_y;
      mine->ph_status = PHHIT2;
      target2->pt_status = PTDET;
   } else {
      mine->ph_target = target->p_no;
      mine->ph_damage = (int)(me->p_ship.s_phaserdamage *
	 (1.0 - (range / (float) myphrange)));
      if (mine->ph_damage < 0)
	 mine->ph_damage = -mine->ph_damage;
      mine->ph_status = PHHIT;

#ifdef BIRD
      if (target->p_flags & PFBIRD) {
	 /* change to PHHIT2 so phaser won't follow bird */
	 mine->ph_status = PHHIT2;
	 mine->ph_x = target->p_x;
	 mine->ph_y = target->p_y;
	 /* xx: slight misuse of fields here */
	 target->p_damage = mine->ph_damage;
	 target->p_whodead = me->p_no;

	 (void) sprintf(buf, "\"AAWWK!\"");
	 warning(1, buf, -1);
      } else
#endif
      {
#ifdef SHORT_PACKETS
	 if(send_short && mine->ph_damage < 1000){
	    swarning(PHASER_HIT_TEXT, (target->p_no & 0x3f)|	
				   (mine->ph_damage & 768) >> 2,
		  (unsigned char)mine->ph_damage & 0xff);
	    return;
	 }
#endif
	 (void) sprintf(buf, "Phaser burst hit %s for %d points",
			target->p_name,
			mine->ph_damage);
	 warning(1, buf, -1);
      }
   }
}
