/*
 * snake.c
 */
#include "copyright.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/time.h>
#include <signal.h>
#include <setjmp.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#ifndef _NO_DEPEND
#include "snake_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern void                   srandom P_((int));
extern pid_t                  getpid P_((void));
extern time_t                 time P_((time_t *));
extern void                   bzero P_((void *, size_t));
extern int                    fprintf P_((FILE *, const char *, ...));
extern int                    isupper P_((int));
extern int                    tolower P_((int));
extern long                   random P_((void));
extern char *                 strncpy P_((char *, const char *, size_t));
extern int                    setitimer P_((int, struct itimerval *, struct itimerval *));
extern void                   perror P_((const char *));
extern int                    sigsetmask P_((int));
extern int                    pause P_((void));

#undef P_
#endif

struct player  *perfs[2];
int             num_perfs = 2;
int             debug = 0;
int             target = -1;
int             berserk = 0;
int             patrol = 0;
int             team1 = 0, team2 = 0;

struct itimerval udt;
int             lastm;
int             tno = 0;

int             diagnolTeams[9] = {0, KLI, ORI, 0, FED, 0, 0, 0, ROM};

void
main(argc, argv)
   int             argc;
   char          **argv;
{
   register int    i;
   int             pno;
   char            tlet;
   int             class = SCOUT;
   char		   buf[3];

   tno = -1;

   srandom(getpid() * time(NULL));
   bzero(perfs, sizeof(perfs));

   for (; argc > 1 && argv[1][0] == '-'; argc--, argv++) {
      switch (argv[1][1]) {
      case 'p':
	 patrol++;
	 break;
      case 'b':
	 berserk++;
	 break;
      case 'd':
	 debug++;
	 break;
      case 'c':		/* ship class */
	 switch (argv[1][2]) {
	 case 'a':
	    class = ASSAULT;
	    break;
	 case 'b':
	    class = BATTLESHIP;
	    break;
	 case 'c':
	    class = CRUISER;
	    break;
	 case 'd':
	    class = DESTROYER;
	    break;
	 case 's':
	    class = SCOUT;
	    break;
	 case 'o':
	    class = STARBASE;
	    break;		/* not fully implemented */
	 default:
	    fprintf(stderr, "Unknown ship class, must be one of [abcdsoA].\n");
	    exit(1);
	    break;
	 }
	 break;

      case 't':{		/* target */
	    char            c;
	    c = argv[1][2];
	    target = -1;
	    if (c == '\0') {
	       fprintf(stderr, "Must specify target.  e.g. -t3.\n");
	       exit(1);
	    }
	    if ((c >= '0') && (c <= '9'))
	       target = c - '0';
	    else if ((c >= 'a') && (c <= 'j'))
	       target = c - 'a' + 10;
	    else {
	       fprintf(stderr, "Must specify target.  e.g. -t3.\n");
	       exit(1);
	    }
	 }
	 break;

      case 'T':		/* team */
	 tlet = argv[1][2];
	 if (isupper(tlet))
	    tlet = tolower(tlet);
	 switch (tlet) {
	 case 'f':
	    if (tno < 0)
	       tno = 0;
	    if (!team1)
	       team1 = FED;
	    else if (!team2)
	       team2 = FED;
	    break;
	 case 'r':
	    if (tno < 0)
	       tno = 1;
	    if (!team1)
	       team1 = ROM;
	    else if (!team2)
	       team2 = ROM;
	    break;
	 case 'k':
	    if (tno < 0)
	       tno = 2;
	    if (!team1)
	       team1 = KLI;
	    else if (!team2)
	       team2 = KLI;
	    break;
	 case 'o':
	    if (tno < 0)
	       tno = 3;
	    if (!team1)
	       team1 = ORI;
	    else if (!team2)
	       team2 = ORI;
	    break;
	 case 'i':		/* means pick random team */
	    if (tno < 0)
	       tno = random() % 4;
#ifdef nodef
	    /* no team boundaries */
	    team1 = 0;
	    team2 = 0;
#endif
	    break;
	 default:
	    fprintf(stderr, "Unknown team type.  Usage -Tx where x is [frko]\n");
	    exit(1);
	 }			/* end switch argv */
	 break;

      default:
	 fprintf(stderr, "Unknown option '%c' (must be one of: dTc)\n",
		 argv[1][1]);
	 exit(1);
      }				/* end switch argv[1][1] */
   }				/* end for */

   /* if -T wasn't specified default to FED */
   if (tno < 0)
      tno = 0;

   /* XX -- teams imply patrol */
   if (team1 && team2)
      patrol++;

   if (team1 && (team2 == diagnolTeams[team1])) {
      /*
       * fprintf(stderr, "snake: can't patrol diagonal teams.\n");
       */
      team2 = 0;
   }
#ifdef nodef
   /* debug */
   if (patrol) {
      printf("snake (%s): patrolling %s,%s\n", teamshort[1 << tno],
	     teamshort[team1],
	     teamshort[team2]);
   }
   fflush(stdout);
#endif

   readsysdefaults();

   (void) signal(SIGTERM, exitSnake);
   (void) signal(SIGHUP, exitSnake);
   (void) signal(SIGINT, exitSnake);
   (void) signal(SIGBUS, exitSnake);
   (void) signal(SIGSEGV, exitSnake);
   openmem();

   lastm = mctl->mc_current;

   for (i = 0; i < 2; i++) {	/* two players per snake */
      pno = findrslot();
      if (pno < 0) {
	 fprintf(stderr, "snake: no room in game\n");
	 if (i > 0)
	    perfs[0]->p_status = PFREE;
	 exit(1);
      }
      me = &players[pno];

      perfs[i] = me;

      me->p_no = pno;
      myship = &me->p_ship;
      mystats = &me->p_stats;

      strcpy(pseudo, "The Snake");
      strcpy(login, "SnkeChrmr");

      strcpy(me->p_name, pseudo);
      me->p_name[sizeof(me->p_name) - 1] = '\0';

      (void) strncpy(me->p_login, login, sizeof(me->p_login));
      me->p_login[sizeof(me->p_login) - 1] = '\0';
      (void) strncpy(me->p_monitor, "Server", sizeof(me->p_monitor));
      me->p_monitor[sizeof(me->p_monitor) - 1] = '\0';
      /*
       * enter(tno, 0, pno, class);
       */
      me->p_team = (1 << tno);
      sprintf(buf, "%c%c", teamlet[me->p_team], shipnos[me->p_no]);
      strncpy(me->p_mapchars, buf, 2);

      config();

      me->p_pos = -1;
      me->p_flags |= PFROBOT;	/* Mark as a robot */
      me->p_flags |= PFSNAKE;	/* Mark as snake */
      if (berserk)
	 me->p_hostile = FED | ROM | ORI | KLI;
   }

   signal(SIGALRM, snakemove);

#ifndef DS
   udt.it_interval.tv_sec = 0;	/* snakes get to move 10/sec */
   udt.it_interval.tv_usec = 100000;
   udt.it_value.tv_sec = 1;
   udt.it_value.tv_usec = 0;

   if (setitimer(ITIMER_REAL, &udt, 0) < 0) {
      perror("setitimer");
      for (i = 0; i < num_perfs; i++) {
	 if (perfs[i])
	    perfs[i]->p_status = PFREE;
      }
      exit(1);
   }
#else
   perfs[0]->p_process = getpid();
   perfs[0]->p_timerdelay = 1;
   perfs[1]->p_process = 0;
   perfs[1]->p_timerdelay = 1;
#endif
   /* allows robots to be forked by the daemon -- Evil ultrix bullshit */
   sigsetmask(0);

   /* NOTE: snakes do not become alive. */

   while (1) {
      pause();
   }
}

int
findtestslot()
{
   register int    i;

   for (i = MAXPLAYER - TESTERS; i < MAXPLAYER; i++) {
      if (players[i].p_status == PFREE) {	/* We have a free slot */
	 players[i].p_status = POUTFIT;	/* possible race code */
	 break;
      }
   }
   if (i == MAXPLAYER) {
      return -1;		/* no room in tester slots */
   }
   bzero(&players[i].p_stats, sizeof(struct stats));
   players[i].p_stats.st_tticks = 1;
   return (i);
}

int
findrslot()
{
   register int    i;

   /* look for tester slot first */
   i = findtestslot();

   if (i > -1)
      return i;

   for (i = 0; i < MAXPLAYER; i++) {
      if (players[i].p_status == PFREE) {	/* We have a free slot */
	 players[i].p_status = POUTFIT;	/* possible race code */
	 break;
      }
   }
   if ((i == MAXPLAYER) || (i == -1)) {
      if (debug) {
	 fprintf(stderr, "No more room in game\n");
      }
      return -1;
   }
   bzero(&players[i].p_stats, sizeof(struct stats));
   players[i].p_stats.st_tticks = 1;
   return (i);
}

void
warning(f, mess)
   int	f;
   char	*mess;
{
   if (debug)
      fprintf(stderr, "warning: %s\n", mess);
}

void
config()
{
   /* mostly not used */
   myship->s_phasercost = 0;
   myship->s_torpcost = 0;
   myship->s_cloakcost = 0;
   myship->s_torpfuse = 1000000;
   myship->s_torpdamage = 10;
   myship->s_plasmadamage = 50;
   myship->s_plasmacost = 0;
   myship->s_plasmaturns = 0;
   myship->s_plasmaspeed = 10;
   myship->s_plasmafuse = 1000000;
   myship->s_wpncoolrate = 100;
   myship->s_egncoolrate = 100;
}
