#include <stdio.h>
#include <varargs.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/file.h>
#include <math.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include <pwd.h>
#include <ctype.h>
#include "defs.h"
#include "struct.h"

#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* players.c */
int openmem P_((void));
void main P_((int argc, char **argv));
char *name_fix P_((char *name));
char *str P_((int n, char *s));
void output P_((...));

#ifdef USE_SYS_PROTOS
extern int shmget P_((key_t, int, int));
extern void perror P_((const char *));
extern int fprintf P_((FILE *, const char *,...));
extern void *shmat P_((int, void *, int));
extern int printf P_((const char *,...));
extern void *malloc P_((size_t));
extern int vfprintf P_((FILE *, const char *, va_list));
extern int vsprintf P_((char *, const char *, va_list));
extern int write P_((int, const void *, unsigned int));
#endif

#undef P_

#define strdup(s)       strcpy((char *)malloc(strlen(s)+1), s)

struct player  *players;
struct player  *me;
struct torp    *torps;
struct ship    *myship;
struct stats   *mystats;
struct planet  *planets;
struct status  *status;
struct phaser  *phasers;
struct message *messages;
struct mctl    *mctl;
struct team    *teams;

int             mode = 0;

int
openmem()
{
   extern int      errno;
   int             shmemKey = PKEY;
   int             shmid;
   struct memory  *sharedMemory;

   errno = 0;
   shmid = shmget(shmemKey, 0, 0);
   if (shmid < 0) {
      if (errno != ENOENT) {
	 perror("shmget");
	 exit(1);
      }
      shmid = shmget(shmemKey, 0, 0);
      if (shmid < 0) {
	 return 0;
      }
   }
   sharedMemory = (struct memory *) shmat(shmid, 0, 0);
   if (sharedMemory == (struct memory *) - 1) {
      perror("shared memory");
      exit(1);
   }
   players = sharedMemory->players;
   torps = sharedMemory->torps;
   planets = sharedMemory->planets;
   status = sharedMemory->status;
   phasers = sharedMemory->phasers;
   mctl = sharedMemory->mctl;
   messages = sharedMemory->messages;
   teams = sharedMemory->teams;

   return 1;
}
char           *ships[] =
{"SC", "DD", "CA", "BB", "AS", "SB", "GA", "AT", "??", "??"};
char	       *ranks[] = 
{"Ensign", "Lieutenant", "Lt. Cmdr.", "Commander", "Captain",
 "Flt. Capt.", "Commodore", "Rear Adm.", "Admiral"};

int             out = 1;	/* stdout for non-socket connections */

void
main(argc, argv)
    int             argc;
    char          **argv;
{
   int             i;
   int             teams[9];

   if (argc > 1)
      out = 0;

   if(!openmem()){
      output("No one playing.\n");
      exit(0);
   }
   for (i = 0; i < 9; i++)
      teams[i] = 0;

   output(
"<>========================================================================<>\n");
   output(
"  Pl: Rank       Name             Login      Display                  Type\n");
   output(
"<>========================================================================<>\n");

   for (i = 0; i < MAXPLAYER; i++) {
      if (players[i].p_status == PFREE)
	 continue;
      output(
"  %c%c: %-10s %-16s %-10s %-24s  %2s\n",

	 players[i].p_mapchars[0], players[i].p_mapchars[1],
	 ranks[players[i].p_stats.st_rank],
	 name_fix(players[i].p_name),
	 str(10,players[i].p_login),
	 str(-24, players[i].p_monitor),
	 ships[players[i].p_ship.s_type]);

      teams[players[i].p_team]++;
   }

   output(
"<>========================================================================<>\n");
   output(
"  Feds: %d   Roms: %d   Kli: %d   Ori: %d\n",
	  teams[FED], teams[ROM], teams[KLI], teams[ORI]);

   if(status->count - status->wait > 0)
      output("  Wait queue: %d\n", status->answer);
   else
      output("  No wait queue.\n");
   output(
"<>========================================================================<>\n");

   exit(0);
}

char *
name_fix(name)

   register 
   char *name;
{
   char	*new = strdup(name);			/* xx, never freed */
   register 
   char *r = new;

   while(*name){
      *r++ = (*name <= 32)?'_':*name;
      name++;
   }
   *r = 0;
   return new;
}

char *
str(n, s)

   int  n;
   register
   char *s;
{
   char                 *new;
   register             i = 0, j;
   register char        *r;

   if(n > 0){
      j = strlen(s);
      if(j <= n)
	 return s;
      else {
	 new = (char *) malloc(n);              /* xx, never freed */
	 r = new;
	 while(*s && i < n){
	    *r++ = *s++;
	    i++;
	 }
	 *r = 0;
	 return new;
      }
   }
   else if(n < 0){
      n = -n;
      j = strlen(s);
      if(j <= n)
         return s;
      else{
         new = (char *) malloc(n);              /* xx, never freed */
         r = new;
         s += (j-n);
         while(*s){
            *r++ = *s++;
         }
         *r = 0;
	 new[0] = '+';	/* indicate that something missing */
	 return new;
      }
   }
   else 
      return NULL;
}

void
output(va_alist)

   va_dcl
{
   va_list	ap;
   char		*format;

   va_start(ap);
   format = va_arg(ap, char *);

   if(out == 1){
      vfprintf(stdout, format, ap);
   }
   else {
      char	buf[512];
      vsprintf(buf, format, ap);
      write(out, buf, strlen(buf));
   }
   va_end(args);
}
