#include <stdio.h>
#include <ctype.h>
#include <strings.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <pwd.h>
#include "defs.h"
#include "struct.h"

#define COMPRESS			"gzip -9"
#define UNCOMPRESS			"gzcat"
#define EXT				".gz"
#define PROG_BACK			"scback"

struct shortstatentry {
   char	name[16], password[16];
   double          st_maxkills; /* max kills ever */
   int             st_tkills;   /* Kills in tournament play */
   int             st_tlosses;  /* Losses in tournament play */
   int             st_tarmsbomb;/* Tournament armies bombed */
   int             st_tplanets; /* Tournament planets conquered */
   int             st_tticks;   /* Tournament ticks */
   /* SB stats are entirely separate */
   int             st_sbkills;  /* Kills as starbase */
   int             st_sblosses; /* Losses as starbase */
   int             st_sbticks;  /* Time as starbase */
   double          st_sbmaxkills;       /* Max kills as starbase */
   int             st_rank;     /* Ranking of the player */
#ifdef BASEPRACTICE
   double          st_dami;     /* damage inflicted */
   double          st_damr;     /* damage received */
#endif
};

#if __STDC__ || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* scback.c */
void main P_((int argc, char **argv));
void preserve P_((char *f));
void convert P_((struct statentry *stat, struct shortstatentry *sstat));
void show P_((struct shortstatentry *sstat));

extern void                   perror P_((const char *));
extern int                    fprintf P_((FILE *, const char *, ...));
extern size_t                 fread P_((void *, size_t, size_t, FILE *));
extern size_t                 fwrite P_((const void *, size_t, size_t, FILE *));
extern int                    fclose P_((FILE *));
extern int                    system P_((const char *));
extern void *                 malloc P_((size_t));
extern int                    link P_((const char *, const char *));
extern int                    unlink P_((const char *));
extern void                   free P_((void *));
extern int                    printf P_((const char *, ...));

#undef P_

void
main(argc, argv)

   int	argc;
   char	**argv;
{
   struct statentry		stat;
   struct shortstatentry	short_stat;
   FILE	*fi, *fo;
   char	buf[80], *name;

   if(!strstr(argv[0], PROG_BACK)){	/* show player */
      if(argc < 3){
	 fprintf(stderr, "usage: %s <playername> <backupfile>\n", argv[0]);
	 exit(0);
      }
      name = argv[1];
      sprintf(buf, "%s %s", UNCOMPRESS, argv[2]);
      fi = popen(buf, "r");
      if(!fi) {
	 perror(argv[2]);
	 exit(1);
      }
      while(fread(&short_stat, sizeof(short_stat), 1, fi)){
	 if(strcmp(short_stat.name, name)==0){
	    show(&short_stat);
	    exit(0);
	 }
      }
      fclose(fi);
      fprintf(stderr, "%s: \"%s\" not found.\n", argv[2], name);
      exit(1);
   }
   else {				/* make backup */

      if(argc < 2){
	 fprintf(stderr, "usage: %s <backupfile>\n", argv[0]);
	 exit(0);
      }

      fi = fopen(PLAYERFILE, "r");
      if(!fi){
	 perror(PLAYERFILE);
	 exit(1);
      }

      preserve(argv[1]);
      fo = fopen(argv[1], "w");
      if(!fo){
	 perror(argv[1]);
	 exit(1);
      }

      while(fread(&stat, sizeof(stat), 1, fi)){
	 convert(&stat, &short_stat);
	 fwrite(&short_stat, sizeof(short_stat), 1, fo);
      }
      fclose(fi);
      fclose(fo);

      sprintf(buf, "%s %s", COMPRESS, argv[1]);
      system(buf);
   }
   exit(0);
}

void
preserve(f)

   char	*f;
{
   char         *backup_f, *ext;
   struct stat  sbuf;

   ext = (char *)malloc(strlen(f)+strlen(EXT)+1);
   if(!ext) { perror("malloc"); exit(1); }
   sprintf(ext, "%s%s", f, EXT);
   
   if(stat(ext, &sbuf) < 0){
      free(ext);
      return;
   }
   backup_f = (char *)malloc(strlen(ext)+2);
   if(!backup_f) { perror("malloc"); exit(1); }
   sprintf(backup_f, "%s~", ext);
   (void)link(ext, backup_f);
   (void)unlink(ext);
   free((char *)backup_f);
   free((char *)ext);
}

void
convert(stat, sstat)
   
   struct statentry		*stat;
   struct shortstatentry	*sstat;
{
   /*
   printf("saving \"%s\"\n", stat->name);
   */
   strcpy(sstat->name, stat->name);
   strcpy(sstat->password, stat->password);

   sstat->st_maxkills = stat->stats.st_maxkills;
   sstat->st_tkills = stat->stats.st_tkills;
   sstat->st_tlosses = stat->stats.st_tlosses;
   sstat->st_tarmsbomb = stat->stats.st_tarmsbomb;
   sstat->st_tplanets = stat->stats.st_tplanets;
   sstat->st_tticks = stat->stats.st_tticks;
   sstat->st_sbkills = stat->stats.st_sbkills;
   sstat->st_sblosses = stat->stats.st_sblosses;
   sstat->st_sbticks = stat->stats.st_sbticks;
   sstat->st_sbmaxkills = stat->stats.st_sbmaxkills;
   sstat->st_rank = stat->stats.st_rank;
#ifdef BASEPRACTICE
   sstat->st_dami = stat->stats.st_dami;
   sstat->st_damr = stat->stats.st_damr;
#endif
}

void
show(sstat)

   struct shortstatentry	*sstat;
{
   printf("%-30s\"%s\"\n", "name:", sstat->name);
   printf("%-30s\"%s\"\n", "password:", sstat->password);
   printf("%-30s%f\n", "st_maxkills:", sstat->st_maxkills);
   printf("%-30s%d\n", "st_tkills:", sstat->st_tkills);
   printf("%-30s%d\n", "st_tlosses:", sstat->st_tlosses);
   printf("%-30s%d\n", "st_tarmsbomb:", sstat->st_tarmsbomb);
   printf("%-30s%d\n", "st_tplanets:", sstat->st_tplanets);
   printf("%-30s%d\n", "st_tticks:", sstat->st_tticks);
   printf("%-30s%d\n", "st_sbkills:", sstat->st_sbkills);
   printf("%-30s%d\n", "st_sblosses:", sstat->st_sblosses);
   printf("%-30s%d\n", "st_sbticks:", sstat->st_sbticks);
   printf("%-30s%f\n", "st_sbmaxkills:", sstat->st_sbmaxkills);
   printf("%-30s%d\n", "st_rank:", sstat->st_rank);
#ifdef BASEPRACTICE
   printf("%-30s%f\n", "st_dami:", sstat->st_dami);
   printf("%-30s%f\n", "st_damr:", sstat->st_damr);
#endif
}
