#include <X11/Xlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/file.h>
#include <math.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include <pwd.h>
#include <string.h>
#include <ctype.h>
#include "defs.h"
#include "struct.h"

#include "planets.h"

/* below */
extern 
struct planet topdata[MAXPLANETS];

struct player *players;
struct player *me;
struct torp *torps;
struct ship *myship;
struct stats *mystats;
struct planet *planets;
struct phaser *phasers;
struct message *messages;
struct mctl *mctl;
struct team *teams;
int restore();

struct planet oldplanets[MAXPLANETS];

restore()
{
   /* restore */
   bcopy(oldplanets, planets, MAXPLANETS * sizeof(struct planet));
   exit(0);
}

main(argc, argv)
   
   int	argc;
   char	**argv;
{
   register i,j,k, team;
   static int teami[4] = { FED, ROM, KLI, ORI };
   int	delay = 500000;

   if(argc > 1)
      delay = atoi(argv[1]);

   openmem();

   /* save old */
   bcopy(planets, oldplanets, MAXPLANETS * sizeof(struct planet));
   srandom(getpid()+time());

   signal(SIGINT, restore);
   for(i=0; i< 1000; i++){
      team = (random()%4);
      j = 0;
      while(j < MAXPLANETS){
	 if((j % 6) == 0) {
	    team ++;
	    if(team > 3)
	       team = 0;
	 }
	 k = topdata[j].pl_no;
	 planets[k].pl_info = (FED|ROM|KLI|ORI);
	 planets[k].pl_owner = teami[team];

	 j ++;
      }
      usleep(delay);
   }
   /* restore */
   bcopy(oldplanets, planets, MAXPLANETS * sizeof(struct planet));
}

openmem()
{
    extern int errno;
    int	shmemKey = PKEY;
    int	shmid;
    struct memory	*sharedMemory;

    errno = 0;
    shmid = shmget(shmemKey, 0, 0);
    if (shmid < 0) {
	if (errno != ENOENT) {
	    perror("shmget");
	    exit(1);
	}
	shmid = shmget(shmemKey, 0, 0);
	if (shmid < 0) {
	    fprintf(stderr, "resetplanets: Daemon not running\n");
	    exit (1);
	}
    }
    sharedMemory = (struct memory *) shmat(shmid, 0, 0);
    if (sharedMemory == (struct memory *) -1) {
	perror("shared memory");
	exit (1);
    }
    players = sharedMemory->players;
    torps = sharedMemory->torps;
    planets = sharedMemory->planets;
    phasers = sharedMemory->phasers;
    mctl = sharedMemory->mctl;
    messages = sharedMemory->messages;
    teams = sharedMemory->teams;
}


/* sorted from top to bottom */

struct planet topdata[MAXPLANETS] = {
    { 29, KLI|PLCORE, KLI, 73000,  5000, "Castor", 6, 30, KLI },
    { 11, ROM, ROM, 45000,  7000, "Eridani", 7, 30, ROM },
    { 15, ROM|PLCORE, ROM, 28000,  8000, "Tauri", 5, 30, ROM },
    { 24, KLI|PLCORE, KLI, 93000,  8000, "Pollux", 6, 30, KLI },
    { 22, KLI, KLI, 60000, 10000, "Andromeda", 9, 30, KLI },
    { 12, ROM|PLCORE, ROM,  4000, 12000, "Aldeberan", 9, 30, ROM },
    { 10, (ROM|PLHOME|PLFUEL|PLREPAIR|PLCORE), ROM, 20000, 20000, "Romulus", 7, 30, ROM },
    { 20, (KLI|PLHOME|PLFUEL|PLREPAIR|PLCORE), KLI, 80000, 20000, "Klingus", 7, 30, KLI },
    { 28, KLI, KLI, 54000, 21000, "Cygni", 5, 30, KLI },
    { 16, ROM|PLCORE, ROM, 28000, 23000, "Draconis", 8, 30, ROM },
    { 17, ROM, ROM, 40000, 25000, "Sirius", 6, 30, ROM },
    { 19, ROM, ROM,  8000, 29000, "Hydrae", 6, 30, ROM },
    { 26, KLI|PLCORE, KLI, 69000, 31000, "Scorpii", 7, 30, KLI },
    { 25, KLI, KLI, 90000, 37000, "Lyrae", 5, 30, KLI },
    { 21, KLI, KLI, 70000, 40000, "Pleiades V", 9, 30, KLI },
    { 23, KLI, KLI, 54000, 40000, "Lalande", 7, 30, KLI },
    { 13, ROM, ROM, 42000, 44000, "Regulus", 7, 30, ROM },
    { 18, ROM, ROM, 25000, 44000, "Indi", 4, 30, ROM },
    { 14, ROM, ROM, 13000, 45000, "Capella", 7, 30, ROM },
    { 27, KLI, KLI, 83000, 48000, "Mira", 4, 30, KLI },
    {  4, FED, FED, 39000, 55000, "Organia", 7, 30, FED },
    { 32, ORI, ORI, 65000, 55000, "El Nath", 7, 30, ORI },
    { 31, ORI, ORI, 92000, 59000, "Cassiopia", 9, 30, ORI },
    {  1, FED, FED, 10000, 60000, "Rigel", 5, 30, FED },
    {  2, FED, FED, 25000, 60000, "Canopus", 7, 30, FED },
    { 33, ORI, ORI, 52000, 60000, "Spica", 5, 30, ORI },
    {  6, FED, FED, 45000, 66000, "Ceti Alpha V", 12, 30, FED },
    { 34, ORI|PLCORE, ORI, 72000, 69000, "Procyon", 7, 30, ORI },
    { 39, ORI|PLCORE, ORI, 85000, 70000, "Antares", 7, 30, ORI },
    {  9, FED, FED, 32000, 74000, "Alpha Centauri", 14, 30, FED },
    {  7, FED|PLCORE, FED, 11000, 75000, "Altair", 6, 30, FED },
    {  0, (FED|PLHOME|PLFUEL|PLREPAIR|PLCORE), FED, 20000, 80000, "Earth", 5, 30, FED },
    { 30, (ORI|PLHOME|PLFUEL|PLREPAIR|PLCORE), ORI, 80000, 80000, "Orion", 5, 30, ORI },
    { 35, ORI, ORI, 64000, 80000, "Polaris", 7, 30, ORI },
    {  3, FED, FED, 44000, 81000, "Beta Crucis", 11, 30, FED },
    { 36, ORI, ORI, 56000, 89000, "Arcturus", 8, 30, ORI },
    {  5, FED|PLCORE, FED, 30000, 90000, "Deneb", 5, 30, FED },
    {  8, FED|PLCORE, FED,  8000, 93000, "Vega", 4, 30, FED },
    { 38, ORI, ORI, 70000, 93000, "Herculis", 8, 30, ORI },
    { 37, ORI|PLCORE, ORI, 91000, 94000, "Ursae Majoris", 13, 30, ORI },
};
