#include <X11/Xlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/file.h>
#include <math.h>
#include <sys/ipc.h>
#include <errno.h>
#include <pwd.h>
#include <ctype.h>
#include "defs.h"
#include "struct.h"

#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* xtkill.c */
void openmem P_((void));
void pmessage P_((char *str, int recip, int group));
void getship P_((struct ship *shipp, int s_type));
void Usage P_((void));

#ifdef USE_SYS_PROTOS
extern int                    shmget P_((key_t, int, int));
extern void                   perror P_((const char *));
extern int                    fprintf P_((FILE *, const char *, ...));
extern void *                 shmat P_((int, void *, int));
extern int                    atoi P_((const char *));
extern void                   bzero P_((void *, size_t));
extern int                    printf P_((const char *, ...));
extern char *                 strncpy P_((char *, const char *, size_t));
#endif

#undef P_


struct status *status;
struct player *players;
struct player *me;
struct torp *torps;
struct ship *myship;
struct stats *mystats;
struct planet *planets;
struct phaser *phasers;
struct message *messages;
struct mctl *mctl;
struct team *teams;

void
openmem()
{
    extern int errno;
    int	shmemKey = PKEY;
    int	shmid;
    struct memory	*sharedMemory;

    errno = 0;
    shmid = shmget(shmemKey, 0, 0);
    if (shmid < 0) {
	if (errno != ENOENT) {
	    perror("shmget");
	    exit(1);
	}
	shmid = shmget(shmemKey, 0, 0);
	if (shmid < 0) {
	    fprintf(stderr, "xtkill: Daemon not running\n");
	    exit (1);
	}
    }
    sharedMemory = (struct memory *) shmat(shmid, 0, 0);
    if (sharedMemory == (struct memory *) -1) {
	perror("shared memory");
	exit (1);
    }
    status = sharedMemory->status;
    players = sharedMemory->players;
    torps = sharedMemory->torps;
    planets = sharedMemory->planets;
    phasers = sharedMemory->phasers;
    mctl = sharedMemory->mctl;
    messages = sharedMemory->messages;
    teams = sharedMemory->teams;
}

char teamlet[] = {'I', 'F', 'R', 'X', 'K', 'X', 'X', 'X', 'O'};
char 	*shipnos="0123456789abcdefghijklmnopqrstuvwxyz";
char *names[] = { "Neutral", "Fed", "Rom", "", "Kli", "", "", "", "Ori"};

void
main(argc, argv)
int argc;
char **argv;
{
    int player;
    char buf[1000];
    int c;

    openmem();
    if(argc == 1) Usage();
    player=atoi(argv[1]);
    if ((player == 0) && (*argv[1] != '0')) {
	c = *argv[1];
	if (c >= 'a' && c <= 'j')
	    player = c - 'a' + 10;
	else {
	    Usage();
	}
    }
    if (players[player].p_status != PALIVE) {
	if (argc > 2)
	    if (*argv[2] == 'F') {
		bzero(&(players[player]), sizeof(struct player)); /* confusion 8/5/91 TC */
		/*players[player].p_status = PFREE;*/
		exit(0);
	    }
	printf("Slot is not alive.\n");
	exit(1);
    }
    if (argc > 2)
	switch (*argv[2]) {
	  case 'T': {
	      int team;
	      
	      switch (argv[2][1]) {
		case 'f': team = 1; break;
		case 'r': team = 2; break;
		case 'k': team = 4; break;
		case 'o': team = 8; break;
		default:  team = 0;
	      }	    
	      players[player].p_team = team;
	      players[player].p_hostile &= ~team;
	      players[player].p_swar &= ~team;
	      sprintf(buf, "GOD->ALL Change:  %2s is now a %s.",
		      players[player].p_mapchars,
		      names[team]);
	      pmessage(buf, 0, MALL);
	      sprintf(buf, "%c%c", teamlet[players[player].p_team],
		      shipnos[player]);
	      strncpy(players[player].p_mapchars, buf, 2);
	  }
	    break;
	  case 'e':
	    sprintf(buf, "GOD->ALL %s (%2s) has been ejected from the game.", 
		    players[player].p_name, players[player].p_mapchars);
	    players[player].p_whydead=KQUIT;
	    players[player].p_explode=10;
	    players[player].p_status=3;
	    players[player].p_whodead=0;
	    pmessage(buf, 0, MALL);
	    break;
	  case 't':
	    switch(argv[2][1]) {
	      case 'f':
		players[player].p_x = planets[0].pl_x;
		players[player].p_y = planets[0].pl_y;
		break;
	      case 'r':
		players[player].p_x = planets[10].pl_x;
		players[player].p_y = planets[10].pl_y;
		break;
	      case 'k':
		players[player].p_x = planets[20].pl_x;
		players[player].p_y = planets[20].pl_y;
		break;
	      case 'o':
		players[player].p_x = planets[30].pl_x;
		players[player].p_y = planets[30].pl_y;
		break;
	      case 'c':
		players[player].p_x = GWIDTH/2;
		players[player].p_y = GWIDTH/2;
		break;
	      default:
		printf("Valid teleports: frkoc.\n");
		exit(1);
	    }
	    break;
	  case 's':
	     switch (argv[2][1]) {
	       case 'a': getship(&players[player].p_ship, ASSAULT); break;
	       case 'b': getship(&players[player].p_ship, BATTLESHIP); break;
	       case 'c': getship(&players[player].p_ship, CRUISER); break;
	       case 'd': getship(&players[player].p_ship, DESTROYER); break;
	       case 's': getship(&players[player].p_ship, SCOUT); break;
	       case 'o': getship(&players[player].p_ship, STARBASE); break;
	       case 'A': getship(&players[player].p_ship, ATT); break;
	       default:
		 printf("Valid ship types: abcdsoA.\n");
		 exit(1);
		 break;
	    }
	    players[player].p_damage = 0;
	    players[player].p_shield = players[player].p_ship.s_maxshield;
	    players[player].p_wtemp = 0;
	    players[player].p_etemp = 0;
	    players[player].p_fuel = players[player].p_ship.s_maxfuel;
	    break;
	  case 'S':		/* super ship */
	    players[player].p_ship.s_maxshield = 750;
	    players[player].p_shield = 750;
	    players[player].p_ship.s_maxdamage = 750;
	    players[player].p_ship.s_maxegntemp = 5000;
	    break;
          case 'W':
	    players[player].p_ship.s_maxwpntemp = 999999;
	    break;
	  case 'D':		/* demote */
	    --players[player].p_stats.st_rank;
	    sprintf(buf, "GOD->ALL %2s was (temporarily) demoted for rank normalization purposes.",
		    players[player].p_mapchars);
	    pmessage(buf, 0, MALL);
	    break;
	  case 'P':
	    ++players[player].p_stats.st_rank;
	    break;
	  case 'k':
	    players[player].p_kills += 1.0;
	    break;
	  case 'a':
	    players[player].p_armies += 6;
	    break;
#ifdef nodef
	  case 'C':
	    teams[players[player].p_team].s_surrender = 6;
	    break;
#endif
	  case 'h':
	    players[player].p_shield = 0;
	    players[player].p_damage = players[player].p_ship.s_maxdamage/2;
	    break;
	  default:
	    Usage();
	}			/* end switch */
    else {
	sprintf(buf, "GOD->ALL %s (%2s) was utterly obliterated.", 
		players[player].p_name, players[player].p_mapchars);
	players[player].p_ship.s_type = STARBASE;
	players[player].p_whydead=KPROVIDENCE;
	players[player].p_explode=10;
	players[player].p_status=3;
	players[player].p_whodead=0;
	pmessage(buf, 0, MALL);
    }
}

void
pmessage(str, recip, group)
char *str;
int recip;
int group;
{
    struct message *cur;
    if (++(mctl->mc_current) >= MAXMESSAGE)
	mctl->mc_current = 0;
    cur = &messages[mctl->mc_current];
    cur->m_no = mctl->mc_current;
    cur->m_flags = group;
    cur->m_time = 0;
    cur->m_from = 255; /* change 3/30/91 TC */
    cur->m_recpt = recip;
    (void) sprintf(cur->m_data, "%s", str);
    cur->m_flags |= MVALID;
}

/* fill in ship characteristics */

void
getship(shipp, s_type)
struct ship *shipp;
int s_type;
{
    switch (s_type) {
    case SCOUT:
	    shipp->s_turns = 570000;
	    shipp->s_accint = 200; /* was 185 */
	    shipp->s_decint = 270; /* was 270 */
	    shipp->s_accs = 100;
	    shipp->s_torpdamage = 25; /* was 25 */
	    shipp->s_plasmadamage = -1;
	    shipp->s_phaserfuse = 10;
	    shipp->s_phaserdamage = 75; /* was 75 */
	    shipp->s_torpspeed = 16; /* was 16 */
	    shipp->s_torpfuse = 16;
	    shipp->s_torpturns = 0;
	    shipp->s_plasmaspeed = 0;
	    shipp->s_plasmafuse = 0;
	    shipp->s_plasmaturns = 0;
	    shipp->s_maxspeed = 12;
	    shipp->s_repair = 80;
	    shipp->s_maxfuel = 5000;
	    shipp->s_detcost = 100;
	    shipp->s_torpcost = 7 * shipp->s_torpdamage;
	    shipp->s_plasmacost = 20 * shipp->s_plasmadamage;
	    shipp->s_phasercost = 7 * shipp->s_phaserdamage;
	    shipp->s_warpcost = 2;
	    shipp->s_cloakcost = 17;
	    shipp->s_recharge = 8;
	    shipp->s_maxarmies = 2;
	    shipp->s_maxshield = 75; /* was 75 */
	    shipp->s_maxdamage = 75;
	    shipp->s_wpncoolrate = 3;
	    shipp->s_egncoolrate = 8;
	    shipp->s_maxwpntemp = 1000;
	    shipp->s_maxegntemp = 1000;
	    shipp->s_type = SCOUT;
	    shipp->s_mass = 1500;
	    shipp->s_tractstr = 2000;
	    shipp->s_tractrng = .7;
	    shipp->s_width = 20;
	    shipp->s_height = 20;
	    break;
    case DESTROYER:
	    shipp->s_turns = 310000;
	    shipp->s_accint = 200;
	    shipp->s_decint = 300;
	    shipp->s_accs = 100;
	    shipp->s_torpdamage = 30;
	    shipp->s_plasmadamage = 75;
	    shipp->s_phaserfuse = 10;
	    shipp->s_phaserdamage = 85;
	    shipp->s_torpspeed = 14;
	    shipp->s_torpfuse = 30;
	    shipp->s_torpturns = 0;
	    shipp->s_plasmaspeed = 15;
	    shipp->s_plasmafuse = 30;
	    shipp->s_plasmaturns = 1;
	    shipp->s_maxspeed = 10;
	    shipp->s_repair = 100;
	    shipp->s_maxfuel = 7000;
	    shipp->s_detcost = 100;
	    shipp->s_torpcost = 7 * shipp->s_torpdamage;
	    shipp->s_plasmacost = 30 * shipp->s_plasmadamage;
	    shipp->s_phasercost = 7 * shipp->s_phaserdamage;
	    shipp->s_warpcost = 3;
	    shipp->s_cloakcost = 21;
	    shipp->s_recharge = 11;
	    shipp->s_maxarmies = 5;
	    shipp->s_maxshield = 85;
	    shipp->s_maxdamage = 85;
	    shipp->s_wpncoolrate = 2;
	    shipp->s_egncoolrate = 7;
	    shipp->s_maxwpntemp = 1000;
	    shipp->s_maxegntemp = 1000;
	    shipp->s_mass = 1800;
	    shipp->s_tractstr = 2500;
	    shipp->s_tractrng = .9;
	    shipp->s_width = 20;
	    shipp->s_height = 20;
	    shipp->s_type = DESTROYER;
	    break;
    case BATTLESHIP: 
     	    shipp->s_turns = 75000; 
            shipp->s_accint = 80;
	    shipp->s_decint = 180;
	    shipp->s_accs = 100;
	    shipp->s_torpdamage = 40;
	    shipp->s_plasmadamage = 130;
	    shipp->s_phaserfuse = 10;
	    shipp->s_phaserdamage = 105;
	    shipp->s_torpspeed = 12;
	    shipp->s_torpfuse = 40;
	    shipp->s_torpturns = 0;
	    shipp->s_plasmaspeed = 15;
	    shipp->s_plasmafuse = 35;
	    shipp->s_plasmaturns = 1;
	    shipp->s_maxspeed = 8;
	    shipp->s_repair = 125;
	    shipp->s_maxfuel = 14000;
	    shipp->s_detcost = 100;
	    shipp->s_torpcost = 9 * shipp->s_torpdamage;
	    shipp->s_plasmacost = 30 * shipp->s_plasmadamage;
	    shipp->s_phasercost = 10 * shipp->s_phaserdamage;
	    shipp->s_warpcost = 6;
	    shipp->s_cloakcost = 30;
	    shipp->s_recharge = 14;
	    shipp->s_maxarmies = 6;
	    shipp->s_maxshield = 130;
	    shipp->s_maxdamage = 130;
	    shipp->s_wpncoolrate = 3;
	    shipp->s_egncoolrate = 6;
	    shipp->s_maxwpntemp = 1000;
	    shipp->s_maxegntemp = 1000;
	    shipp->s_mass = 2300;
	    shipp->s_tractstr = 3700;
	    shipp->s_tractrng = 1.2;
	    shipp->s_width = 20;
	    shipp->s_height = 20;
	    shipp->s_type = BATTLESHIP;
	    break;
    case ASSAULT:
	    shipp->s_turns = 120000;
	    shipp->s_accint = 100;
	    shipp->s_decint = 200;
	    shipp->s_accs = 100;
	    shipp->s_torpdamage = 30;
	    shipp->s_plasmadamage = -1;
	    shipp->s_phaserfuse = 10;
	    shipp->s_phaserdamage = 80;
	    shipp->s_torpspeed = 16;
	    shipp->s_torpfuse = 30;
	    shipp->s_torpturns = 0;
	    shipp->s_plasmaspeed = 0;
	    shipp->s_plasmafuse = 0;
	    shipp->s_plasmaturns = 0;
	    shipp->s_maxspeed = 8;
	    shipp->s_repair = 120;
	    shipp->s_maxfuel = 6000;
	    shipp->s_detcost = 100;
	    shipp->s_torpcost = 9 * shipp->s_torpdamage;
	    shipp->s_plasmacost = 20 * shipp->s_plasmadamage;
	    shipp->s_phasercost = 7 * shipp->s_phaserdamage;
	    shipp->s_warpcost = 3;
	    shipp->s_cloakcost = 17;
	    shipp->s_recharge = 10;
	    shipp->s_maxarmies = 20;
	    shipp->s_maxshield = 80;
	    shipp->s_maxdamage = 200;
	    shipp->s_wpncoolrate = 2;
	    shipp->s_egncoolrate = 6;
	    shipp->s_maxwpntemp = 1000;
	    shipp->s_maxegntemp = 1200;
	    shipp->s_mass = 2300;
	    shipp->s_tractstr = 2500;
	    shipp->s_tractrng = .7;
	    shipp->s_width = 20;
	    shipp->s_height = 20;
	    shipp->s_type = ASSAULT;
	    break;
    case STARBASE:
	    shipp->s_turns = 50000;
	    shipp->s_accint = 100;
	    shipp->s_decint = 200;
	    shipp->s_accs = 100;
	    shipp->s_torpdamage = 30;
	    shipp->s_plasmadamage = 150;
	    shipp->s_phaserfuse = 4;
	    shipp->s_phaserdamage = 120;
	    shipp->s_torpspeed = 14;
	    shipp->s_torpfuse = 30;
	    shipp->s_torpturns = 0;
	    shipp->s_plasmaspeed = 15;
	    shipp->s_plasmafuse = 25;
	    shipp->s_plasmaturns = 1;
	    shipp->s_repair = 140;
	    shipp->s_maxfuel = 60000;
	    shipp->s_detcost = 100;
	    shipp->s_torpcost = 10 * shipp->s_torpdamage;
	    shipp->s_plasmacost = 25 * shipp->s_plasmadamage;
	    shipp->s_phasercost = 8 * shipp->s_phaserdamage;
	    shipp->s_warpcost = 10;
	    shipp->s_cloakcost = 75;
	    shipp->s_recharge = 35;
	    shipp->s_maxarmies = 25;
	    shipp->s_maxshield = 500;
	    shipp->s_maxdamage = 600;
	    shipp->s_wpncoolrate = 4;
	    shipp->s_egncoolrate = 4;
	    shipp->s_maxspeed = 2;
	    shipp->s_maxwpntemp = 1300;
	    shipp->s_maxegntemp = 1000;
	    shipp->s_mass = 5000;
	    shipp->s_tractstr = 8000;
	    shipp->s_tractrng = 1.5;
	    shipp->s_width = 20;
	    shipp->s_height = 20;
	    shipp->s_type = STARBASE;
	    break;
    case ATT:
	    shipp->s_turns = 2140000000;
	    shipp->s_accint = 3000;
	    shipp->s_decint = 3000;
	    shipp->s_accs = 100;
	    shipp->s_torpdamage = 1000;
	    shipp->s_plasmadamage = 1000;
	    shipp->s_phaserfuse = 2;
	    shipp->s_phaserdamage = 10000;
	    shipp->s_torpspeed = 30;
	    shipp->s_torpfuse = 200;
	    shipp->s_torpturns = 10;
	    shipp->s_plasmaspeed = 45;
	    shipp->s_plasmafuse = 10000;
	    shipp->s_plasmaturns = 14;
	    shipp->s_maxspeed = 60;
	    shipp->s_repair = 30000;
	    shipp->s_maxfuel = 60000;
	    shipp->s_detcost = 1;
	    shipp->s_torpcost = 1;
	    shipp->s_plasmacost = 1;
	    shipp->s_phasercost = 1;
	    shipp->s_warpcost = 1;
	    shipp->s_cloakcost = 1;
	    shipp->s_recharge = 1000;
	    shipp->s_maxarmies = 1000;
	    shipp->s_maxshield = 30000;
	    shipp->s_maxdamage = 30000;
	    shipp->s_wpncoolrate = 100;
	    shipp->s_egncoolrate = 100;
	    shipp->s_maxwpntemp = 10000;
	    shipp->s_maxegntemp = 10000;
	    shipp->s_mass = 32765;
	    shipp->s_tractstr = 32765;
	    shipp->s_tractrng = 25;
	    shipp->s_width = 28;
	    shipp->s_height = 28;
	    shipp->s_type = ATT;
	    break;
    default:
	    shipp->s_turns = 170000;
	    shipp->s_accint = 150;
	    shipp->s_decint = 200;
	    shipp->s_accs = 100;
	    shipp->s_torpdamage = 40;
	    shipp->s_plasmadamage = 100;
	    shipp->s_phaserfuse = 10;
	    shipp->s_phaserdamage = 100;
	    shipp->s_torpspeed = 12;
	    shipp->s_torpfuse = 40;
	    shipp->s_torpturns = 0;
	    shipp->s_plasmaspeed = 15;
	    shipp->s_plasmafuse = 35;
	    shipp->s_plasmaturns = 1;
	    shipp->s_maxspeed = 9;
	    shipp->s_repair = 110;
	    shipp->s_maxfuel = 10000;
	    shipp->s_detcost = 100;
	    shipp->s_torpcost = 7 * shipp->s_torpdamage;
	    shipp->s_plasmacost = 30 * shipp->s_plasmadamage;
	    shipp->s_phasercost = 7 * shipp->s_phaserdamage;
	    shipp->s_warpcost = 4;
	    shipp->s_cloakcost = 26;
	    shipp->s_recharge = 12;
	    shipp->s_maxarmies = 10;
	    shipp->s_maxshield = 100;
	    shipp->s_maxdamage = 100;
	    shipp->s_wpncoolrate = 2;
	    shipp->s_egncoolrate = 6;
	    shipp->s_maxwpntemp = 1000;
	    shipp->s_maxegntemp = 1000;
	    shipp->s_mass = 2000;
	    shipp->s_tractstr = 3000;
	    shipp->s_tractrng = 1.0;
	    shipp->s_width = 20;
	    shipp->s_height = 20;
	    shipp->s_type = CRUISER;
	    break;
    }
}

void
Usage()
{
    printf("xtkill [0-9a-j] <mode><mode option>");
    printf("Valid modes: e(ject), s(hip class)[abcosA], t(eleport)[frkoc],");
    printf("  S(uper), T(eam)[frko], D(emote), P(romote), F(ree slot), k(ills increment),");
    printf("  h(arm) (no mode == obliterate). W(eapon temp infinite)\n");
    exit(1);
}

